<?php

// $Id: skeleton_instance.inc,v 1.23 2009/08/27 17:36:57 deviantintegral Exp $

/**
 * @file
 *   Functions for defining an instance of a book generated by the skeleton module
 */

include_once('skeleton_common.inc');

/**
 * Show the templates assigned to a skeleton and those still available.
 *
 * @param $skeleton
 *   An object from the {skeleton} table, indicating the outline to act upon.
 * @return $output
 *   Themed HTML, including the outline form
 */
function skeleton_define($skeleton) {
  $result = db_query("SELECT COUNT(1) FROM {skeleton_template}");
  if (db_result($result) == 0) {
    $output .= t('<p>No skeleton templates are available to add to this outline. <a href="@template-add-url">Add a skeleton template</a>.</p>', array('@template-add-url' => url('admin/content/skeleton/template/add')));
    return $output;
  }
  $templates = skeleton_get_tree($skeleton->skeleton_id);
  $output .= drupal_get_form('skeleton_define_form', $skeleton, $templates);
  return $output;
}

/**
 * Callback to add a template to a skeleton
 *
 * @param $action
 *   Determines what to do with the template (add / remove).
 * @param $skeleton
 *   An object from the {skeleton} table, indicating the outline to act upon
 * @param $template
 *   An object from the {skeleton_template} table, indicating the template to act upon
 */
function skeleton_assign($action, $skeleton, $template) {
  $check = db_result(db_query("SELECT COUNT(*) FROM {skeleton_data} WHERE skeleton_id = %d AND template_id = %d", $skeleton->skeleton_id, $template->template_id));
  if (!$check && $action == 'add') {
    $weight = 0;
    $parent = 0;
    // TODO -- calculate the parent and weight correctly?
    db_query("INSERT INTO {skeleton_data} (skeleton_id, template_id, parent, weight) VALUES (%d, %d, %d, %d)", $skeleton->skeleton_id, $template->template_id, $parent, $weight);
    drupal_set_message(t('Template assigned'));
    drupal_goto('admin/content/skeleton/skeleton/'. $skeleton->skeleton_id . '/edit');
  }
  elseif ($check && $action == 'remove') {
    // update the children with the new parent, keep the original weight
    if ($template->parent) {
      $parent = $template->parent;
    }
    else {
      $parent = 0;
    }
    db_query("UPDATE {skeleton_data} SET parent = %d WHERE skeleton_id = %d AND parent = %d", $parent, $skeleton->skeleton_id, $template->template_id);

    // remove
    db_query("DELETE FROM {skeleton_data} WHERE skeleton_id = %d AND template_id = %d", $skeleton->skeleton_id, $template->template_id);
    drupal_set_message(t('Template removed'));
    drupal_goto('admin/content/skeleton/skeleton/'. $skeleton->skeleton_id . '/edit');
  }
  else {
    return 'failed';
  }
}

/**
 * FormsAPI for skeleton_define()
 * @param $skeleton
 *   An object from the {skeleton} table, indicating the outline to act upon
 * @param $templates
 *   The template tree for this outline, generated by skeleton_get_tree()
 * @return $form
 *   A functional $form array.
 */
function skeleton_define_form($form_state, $skeleton, $templates) {
  $form = array();
  $form['#tree'] = TRUE;
  $form['skeleton_id'] = array(
    '#type' => 'value',
    '#value' => $skeleton->skeleton_id,
  );
  // show the remaining options
  $remove = array();
  foreach ($templates as $i => $template) {
    $remove[] = $template->template_id;
  }
  if (!empty($remove)) {
    $result = db_query("SELECT template_id, template, node_type, node_data FROM {skeleton_template} WHERE template_id NOT IN (". implode(', ', $remove) .") ORDER BY template_id");
  }
  else {
    $result = db_query("SELECT template_id, template, node_type, node_data FROM {skeleton_template} ORDER BY template_id");
  }
  while ($template = db_fetch_object($result)) {
    $form['available_templates'][$template->template_id] = array(
      'template_id' => array(
        '#type' => 'value',
        '#value' => $template->template_id,
      ),
      'template' => array(
        '#value' => check_plain($template->template),
      ),
      'node_type' => array(
        '#value' => $template-> node_type,
      ),
      'view' => array(
        '#value' => l(t('view template'), 'admin/content/skeleton/template/'. $template->template_id . '/view'),
      ),
      'modify' => array(
        '#value' => l(t('edit template'), 'admin/content/skeleton/template/'. $template->template_id . '/edit', array('query' => drupal_get_destination())),
      ),
      'add' => array(
        '#value' => l(t('add template to skeleton'), 'admin/content/skeleton/assign/add/'. $skeleton->skeleton_id .'/'. $template->template_id),
      ),
    );
    if (module_exists('translation')) {
      $form['multilang'] = array(
        '#type' => 'value',
        '#value' => TRUE,
      );
      $node = (object)unserialize($template->node_data);
      $form['available_templates'][$template->template_id]['language'] = array(
        '#value' => locale_language_name($node->language),
      );
    }
  }
  if (empty($form['available_templates'])) {
    $form['available_templates'] = array(
      '#prefix' => '<p>',
      '#suffix' => '</p>',
      '#value' => t('All available templates have been assigned to this skeleton outline.'),
    );
  }
  $form['available_templates']['create'] = array(
    '#value' => '<p>'. l(t('Create new template'), 'admin/content/skeleton/template/add') .'</p>',
  );
  
  if (count($templates) > 0) {
    $form['current_templates']['help_text'] = array(
      '#prefix' => '<p>',
      '#suffix' => '</p>',
      '#value' => t('Below are the current templates assigned to this skeleton outline.'),
    );
    $form['current_templates']['notes'] = array(
      '#prefix' => '<div class="form-item"><p class="description">',
      '#suffix' => '</p></div>',
      '#value' => t('If you change a template parent, all the children of that parent will be reassigned as well.'),
    );
  }
  else {
    $form['current_templates'] = array(
      '#prefix' => '<p>',
      '#suffix' => '</p>',
      '#value' => t('No templates are currently assigned to this skeleton outline.'),
    );
  }
  foreach ($templates as $i => $template) {
    $options = array(0 => '<none>');
    foreach ($templates as $temp) {
      if ($temp->template_id != $template->template_id && (empty($template->children) || !in_array($temp->template_id, $template->children))) {
        $options[$temp->template_id] = $temp->template;
      }
    }
    $form['current_templates'][$i]['template_id'] = array(
      '#type' => 'value',
      '#value' => $template->template_id,
    );
    $form['current_templates'][$i]['template'] = array(
      '#value' => str_repeat('-', $template->depth) .' '. check_plain($template->template),
    );
    $form['current_templates'][$i]['node_type'] = array(
      '#value' => check_plain($template->node_type),
    );
    if (module_exists('translation')) {
      $form['multilang'] = array(
        '#type' => 'value',
        '#value' => TRUE,
      );
      $node = (object)unserialize($template->node_data);
      $form['current_templates'][$i]['language'] = array(
        '#value' => locale_language_name($node->language),
      );
    }
    $form['current_templates'][$i]['parent'] = array(
      '#type' => 'select',
      '#default_value' => $template->parent,
      '#options' => $options,
    );
    $form['current_templates'][$i]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $template->weight,
      '#delta' => 15
    );
    $form['current_templates'][$i]['view'] = array(
      '#value' => l(t('view template'), 'admin/content/skeleton/template/'. $template->template_id . '/view'),
    );
    $form['current_templates'][$i]['modify'] = array(
      '#value' => l(t('edit template'), 'admin/content/skeleton/template/'. $template->template_id . '/edit', array('query' => drupal_get_destination())),
    );
    $form['current_templates'][$i]['delete'] = array(
      '#value' => l(t('remove template from skeleton'), 'admin/content/skeleton/assign/remove/'. $skeleton->skeleton_id .'/'. $template->template_id)
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 100,
  );
  $form['create'] = array(
    '#type' => 'submit',
    '#value' => t('Create instance'),
    '#submit' => array('skeleton_create_redirect'),
    '#access' => _skeleton_user_can_create_instance(),
    '#weight' => 150,
  );
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#submit' => array('skeleton_delete_redirect'),
    '#weight' => 200,
  );
  return $form;
}

/**
 * FormsAPI for skeleton_define()
 */
function skeleton_define_form_submit($form, &$form_state) {
  foreach ($form_state['values']['current_templates'] as $key => $template) {
    // only the numeric keys are templates, so process those
    if (is_numeric($key)) {
      db_query("UPDATE {skeleton_data} SET parent = %d, weight = %d WHERE skeleton_id = %d AND template_id = %d", $template['parent'], $template['weight'], $form_state['values']['skeleton_id'], $template['template_id']);
    }
  }
  drupal_set_message(t('Skeleton changes saved.'));
}

/**
 * FormsAPI for skeleton_define()
 * Formats the form in a table.
 */
function theme_skeleton_define_form($form) {
  // Build current templates table.
  $rows = array();
  foreach (element_children($form['current_templates']) as $i) {
    $template = &$form['current_templates'][$i];
    if (isset($template['template_id'])) {
      $row = array(
        drupal_render($template['template']),
        drupal_render($template['node_type']),
      );
      if (!empty($form['multilang'])) {
        $row[] = drupal_render($template['language']);
      }
      $row[] = drupal_render($template['parent']);
      $row[] = drupal_render($template['weight']);
      $row[] = theme('item_list', array(drupal_render($template['view']), drupal_render($template['modify']), drupal_render($template['delete'])));
      $rows[] = $row;
    }
  }
  $output = '<h4>' . t('Current templates') . '</h4>';
  if (!empty($rows)) {
    // Finish table.
    $header = array(t('Template'), t('Node type'));
    if ($form['multilang']) {
      $header[] = t('Language');
    }
    $header[] = t('Parent');
    $header[] = t('Weight');
    $header[] = array('data' => t('Operations'), 'colspan' => 2);
    $output .= drupal_render($form['current_templates']['help_text']);
    $output .= theme('table', $header, $rows, array('id' => 'skeleton'));
  }
  
  // Add anything under current_templates.
  $output .= drupal_render($form['current_templates']);
  
  // Build available templates table.
  $rows = array();
  foreach (element_children($form['available_templates']) as $i) {
    $template = &$form['available_templates'][$i];
    if (isset($template['template_id'])) {
      $row = array(
        drupal_render($template['template']),
        drupal_render($template['node_type'])
      );
      if (!empty($form['multilang'])) {
        $row[] = drupal_render($template['language']);
      }
      $row[] = theme('item_list', array(drupal_render($template['view']), drupal_render($template['modify']), drupal_render($template['add'])));
      $rows[] = $row;
    }
  }
  $output .= '<h4>' . t('Available templates') . '</h4>';
  if (!empty($rows)) {
    $header = array(t('Name'), t('Node type'));
    if ($form['multilang']) {
      $header[] = t('Language');
    }
    $header[] = t('Operations');
    $output .= drupal_render($form['available_templates']['help_text']);
    $output .= theme('table', $header, $rows);
  }
  
  // Append any additional form elements to the bottom of the form.
  $output .= drupal_render($form);
  return $output;
}

/**
 * Redirect when clicking delete button on the skeleton form.
 */
function skeleton_delete_redirect($form_id, &$form_state) {
  $form_state['redirect'] = 'admin/content/skeleton/skeleton/' . $form_state['values']['skeleton_id'] . '/delete';
}

/**
 * Redirect when clicking "Create instance" on the skeleton form.
 */
function skeleton_create_redirect($form_id, &$form_state) {
  $form_state['redirect'] = 'admin/content/skeleton/skeleton/'. $form_state['values']['skeleton_id'] . '/create';
}

/**
 * Edit a skeleton definition
 *
 * @param $skeleton
 *   An object from the {skeleton} table, indicating the outline to act upon.
 * @return
 *   Themed HTML, including the outline form
 */
function skeleton_edit_instance($skeleton) {
  $output = '';
  drupal_set_title(check_plain($skeleton->skeleton));
  $output .= '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  include_once('skeleton_instance.inc');
  return skeleton_define($skeleton);
}

/**
 * Delete a skeleton
 *
 * @param $skeleton
 *   An object from the {skeleton} table, indicating the outline to act upon.
 * @return
 *   Themed HTML, including the deletion form
 */
function skeleton_delete_instance($skeleton) {
  $output = '';
  drupal_set_title(t('Skeleton: Delete'));
  $output .= '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  $output .= t('<p>Are you sure you want to delete this skeleton outline?</p>');
  $output .= drupal_get_form('skeleton_delete_form', $skeleton->skeleton_id);
  return $output;
}

/**
 * FormsAPI for skeleton_delete_instance()
 */
function skeleton_delete_form($form_state, $skeleton_id) {
  $form = array();
  $form['skeleton_id'] = array('#type' => 'value', '#value' => $skeleton_id);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete skeleton'),
    '#prefix' => l(t('Cancel action'), 'admin/content/skeleton') .'<br />'
  );
  return $form;
}

/**
  * FormsAPI for skeleton_delete_form()
  */
function skeleton_delete_form_submit($form, &$form_state) {
  db_query("DELETE from {skeleton} WHERE skeleton_id = %d", $form_state['values']['skeleton_id']);
  // We rebuild the menu system to update the links at node/add.
  menu_rebuild();
  // TODO add delete for legacy items
  drupal_set_message(t('Skeleton deleted.'));
  drupal_goto('admin/content/skeleton');
}

/**
 * Create a new instance of a skeleton
 * This function generates the actual book pages.
 *
 * @param $skeleton
 *   An object from the {skeleton} table, indicating the outline to act upon.
 * @return
 *   Themed HTML, including the instance creation form
 */
function skeleton_create_instance($skeleton) {
  $output = '';
  drupal_set_title(t('Skeleton: Create'));
  $output .= '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  $tree = skeleton_get_tree($skeleton->skeleton_id);
  if (empty($tree)) {
    $output .= '<p>' . t('You must first assign at least one template to this skeleton outline. <a href="@edit-url">Edit this skeleton</a> to add a template to it.', array('@edit-url' => url('admin/content/skeleton/skeleton/' . $skeleton->skeleton_id . '/edit'))) . '</p>';
    return $output;
  }
  drupal_add_js(drupal_get_path('module', 'skeleton') . '/skeleton-admin.js');
  $output .= drupal_get_form('skeleton_create_instance_form', $skeleton);
  return $output;
}

/**
 * FormsAPI for skeleton_create_instance()
 */
function skeleton_create_instance_form(&$form_state, $skeleton) {
  $form = array();
  $form['instance'] = array(
    '#type' => 'fieldset',
    '#title' => t('Skeleton Information'),
    '#collapsible' => TRUE
  );

  // Count the number of items with the parent set to 0. If this is more than
  // one, we can't have a template as the book introduction.
  $templates = skeleton_get_tree($skeleton->skeleton_id);
  $parents_count = 0;
  foreach($templates as $template) {
    if ($template->parents[0] == 0) {
      $parents_count++;
    }
    if ($parents_count > 1) {
      $form['instance']['introduction_enabled'] = array(
        '#type' => 'value',
        '#value' => TRUE,
      );
      break;
    }
  }
  $form['instance']['author'] = array(
    '#type' => 'textfield',
    '#title' => t('Book author'),
    '#description' => t('Select an author for the created book, or leave blank to set yourself as the author.'),
    '#access' => user_access('administer nodes'),
    '#default_value' => $form_values['author'],
    '#size' => 40,
    '#autocomplete_path' => 'user/autocomplete',
    '#weight' => -50,
  );
  if ($parents_count < 2 && (!$form_state['add_introduction'])) {
    $form['instance']['add_introduction'] = array(
      '#type' => 'submit',
      '#value' => t('Toggle custom introduction for this book'),
      '#description' => t('This will add an introduction page to this book, and all templates will be made a child of the introduction page.'),
      '#ahah' => array(
        'path' => 'skeleton/introduction',
        'effect' => 'slide',
        'wrapper' => 'introduction-wrapper',
      ),
      '#submit' => array('skeleton_add_introduction_submit'), // If no javascript action.
      '#weight' => 100,
    );
    $form['instance']['introduction_wrapper'] = array(
      '#value' => '<div id="introduction-wrapper" /></div>',
      '#weight' => -25,
    );
  }
  else {
    $form['instance'] += skeleton_introduction_form($form_state);
  }
  $form['instance']['skeleton_id'] = array(
    '#type' => 'value',
    '#value' => $skeleton->skeleton_id,
  );

  // We need to determine if any skeleton tokens are used in this skeleton
  // instance.
  $skeleton_tokens = array();
  $result = db_query("SELECT * FROM {skeleton_token}");
  while ($token = db_fetch_object($result)) {
    $token->description = check_plain($token->description);
    $skeleton_tokens[] = $token;
  }

  // For each template, find any tokens used and put them in $required_tokens.
  $required_tokens = array();
  foreach ($templates as $template_data) {
    $node = (object)unserialize($template_data->node_data);
    foreach ($skeleton_tokens as $token) {
      if (strstr($node->title, $token->token) || strstr($node->body, $token->token)) {
        $required_tokens[] = $token;
      }
    }
  }

  // If there are required tokens, generate the form.
  if (!empty($required_tokens)) {
    $form['skeleton_tokens'] = array(
      '#type' => 'fieldset',
      '#title' => t('Skeleton tokens'),
      '#description' => t('The following tokens are used in this skeleton outline. Please provide values for the following tokens.'),
      '#collapsible' => TRUE,
      '#tree' => TRUE,
    );
    foreach ($required_tokens as $token) {
      $form['skeleton_tokens'][$token->token] = array(
        '#type' => 'textfield',
        '#title' => $token->token,
        '#maxlength' => 384,
        '#description' => $token->description,
        '#default_value' => isset($form_state['values']['skeleton_tokens'][$token->token]) ? $form_state['values']['skeleton_tokens'][$token->token] : "",
      );
    }
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create new book from skeleton'),
  );
  return $form;
}

/**
 * Forms API validation function for the create instance form. We need this so
 * we don't throw errors for required fields when changing the introduction
 * status. A better way would be a way to intercept / override #required in
 * specific cases, but the Forms API doesn't support that.
 * 
 * @param $form
 *   The form being validated.
 * @param $form_state
 *   The current state of the form.
 */
function skeleton_create_instance_form_validate($form, &$form_state) {
  if (!($form_state['clicked_button']['#value'] == t('Create new book from skeleton'))) {
    return;
  }
  if (!empty($form_state['values']['author'])) {
    if (!user_load(array('name' => $form_state['values']['author']))) {
      form_set_error('author', t('The account %user you selected as the author does not exist.', array('%user' => $form_state['values']['author'])));
    }
  }
  if (isset($form['instance']['title'])) {
    if (empty($form_state['values']['title'])) {
      form_set_error('title', t('You must set a title for the custom introduction.'));
    }
    if (empty($form_state['values']['body'])) {
      form_set_error('body', t('You must set a body for the custom introduction.'));
    }
  }
  // Now, do token validation.
  foreach(element_children($form['skeleton_tokens']) as $token) {
    if (empty($form_state['values']['skeleton_tokens'][$token])) {
      form_set_error("skeleton_tokens][$token", t('You must set a value for the %token token.', array('%token' => $token)));
    }
  }
}

/**
 * Return a Form API array of the introduction form.
 *
 * @param $form_state
 *   The current state of the form.
 * @return
 *   The Form API array.
 */
function skeleton_introduction_form($form_state) {
  // Add a remove button for the JS-disabled case.
  if ($form_state['add_introduction']) {
    $form['remove_introduction'] = array(
      '#type' => 'submit',
      '#value' => t('Remove introduction to this book'),
      '#submit' => array('skeleton_add_introduction_submit'), // If no javascript action.
      '#weight' => -100,
    );
  }
  $form['title'] = array(
    '#type' => 'textfield',
    '#size' => 40,
    '#title' => t('Book title'),
    '#description' => t('The title of the new book you wish to create'),
    '#weight' => -10,
  );
  if (module_exists('translation')) {
    // We are always creating a book page. This is modified from
    // locale_form_alter().
    if (variable_get('language_content_type_book', 0)) {
      $form['language'] = array(
        '#type' => 'select',
        '#title' => t('Language'),
        '#options' => array('' => t('Language neutral')) + locale_language_list('name'),
        '#weight' => -5,
      );
    }
    // Node type without language selector: assign the default for new nodes
    else {
      $default = language_default();
      $form['language'] = array(
        '#type' => 'value',
        '#value' => $default->language
      );
    }
  }
  $form['body'] = array(
    '#type' => 'textarea',
    '#cols' => 40,
    '#rows' => 10,
    '#title' => t('Book introduction'),
    '#description' => t('A description of the contents of this book.'),
    '#suffix' => t('<p>After you create this instance, you will be able to edit the complete book page.</p>'),
    '#weight' => 0,
  );
  
  return $form;
}

/**
 * Submit handler to enable a custom introduction to a book. This handler is
 * used when javascript is not available. It makes changes to the form state
 * and the entire form is rebuilt during the page reload.
 */
function skeleton_add_introduction_submit($form, &$form_state) {
  // Set the form to rebuild and run submit handlers.
  // Make the changes we want to the form state.
  if ($form_state['values']['add_introduction']) {
    $form_state['add_introduction'] = TRUE;
  }
  $form_state['rebuild'] = TRUE;
  $form = form_builder('skeleton_create_instance_form', $form, $form_state);
}

/**
 * Enable or disable a custom introduction form for an instantiated book.
 */
function skeleton_introduction_js() {
  // Build the new form
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  // Add the new element to the stored form. Without adding the element to the
  // form, Drupal is not aware of this new elements existence and will not
  // process it. We retreive the cached form, add the element, and resave.
  if (!$form = form_get_cache($form_build_id, $form_state)) {
    exit();
  }
  // Get the form element
  if (!$form['instance']['introduction_added']) {
    $form_element = skeleton_introduction_form($form_id);
    // Let other modules alter it.
    drupal_alter('form', $form_element, array(), 'skeleton_introduction_js');
    $form['instance'] += $form_element;
    $form['instance']['introduction_added'] = array(
      '#type' => 'value',
      '#value' => TRUE,
    );
  }
  else {
    unset($form['instance']['title']);
    unset($form['instance']['body']);
    unset($form['instance']['language']);
    unset($form['instance']['introduction_added']);
  }
  form_set_cache($form_build_id, $form, $form_state);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );

  // Rebuild the form.
  $form = form_builder('skeleton_create_instance_form', $form, $form_state);

  // Render the new output. 
  $introduction_form = $form['instance'];
  unset($introduction_form['add_introduction']);
  unset($introduction_form['author']);
  unset($introduction_form['#type']);
  $output = drupal_render($introduction_form);
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
 * FormsAPI for skeleton_create_instance()
 * Creates the nodes from the outline using drupal_execute().
 *
 * Note: The nid fetching sequence is sketchy and might need rewriting
 * 
 */
function skeleton_create_instance_form_submit($form, &$form_state) {
  module_load_include('inc', 'node', 'node.pages');
  module_load_include('inc', 'skeleton', 'skeleton_token');

  // Use the author as specified, otherwise default to the current user.
  if (!empty($form_state['values']['author'])) {
    $user = user_load(array('name' => $form_state['values']['author']));
  }
  else {
    global $user;
  }

  $nodes = skeleton_get_tree($form_state['values']['skeleton_id']);
  if (!empty($nodes)) {
    $parents = array();
    // If we are not using a custom introduction, generate the first page from
    // the appropriate template.
    if (!$form_state['values']['title']) {
      // Get the first template, use it's data.
      $node = array();
      $node['values'] = unserialize($nodes[0]->node_data);

      // Pathauto provides a 'bookpathalias' token which is very useful for
      // generating inline URL's. However, the path isn't generated until the
      // node is saved. Escape out the token if it exists, and replace it later
      // after the node is saved for the first time.
      if (module_exists('pathauto')) {
        $node['values']['title'] = str_replace('[bookpathalias]', '[\skeleton-bookpathalias\]', $node['values']['title']);
        $node['values']['body'] = str_replace('[bookpathalias]', '[\skeleton-bookpathalias\]', $node['values']['body']);
        // Also do any CCK text fields.
        $content_type = content_types($node['values']['type']);
        $fields = $content_type['fields'];
        foreach ($fields as $field) {
          if ($field['type'] == 'text' && is_array($node['values'][$field['field_name']])) {
            foreach (array_keys($node['values'][$field['field_name']]) as $delta) {
              $node['values'][$field['field_name']][$delta]['value'] = str_replace('[bookpathalias]', '[\skeleton-bookpathalias\]', $node['values'][$field['field_name']][$delta]['value']);
            }
          }
        }
      }

      // Replace custom tokens in the title and body fields. Replace skeleton
      // tokens first so they may be used as node tokens such as [title].
      if (!empty($form_state['values']['skeleton_tokens'])) {
        $node['values'] = (array)_skeleton_token_replace_values((object)$node['values'], 'skeleton', $form_state['values']['skeleton_tokens']);
      }

      $node['values'] = (array)_skeleton_token_replace_values((object)$node['values'], 'node', (object)$node['values']);
      $form_state['values']['title'] = $node['values']['title'];
      $form_state['values']['body'] = $node['values']['body'];

      $node['values']['tokens'] = $form_state['values']['skeleton_tokens'];
      $form_state['values']['tokens'] = $form_state['values']['skeleton_tokens'];

      $form_state['values']['language'] = $node['values']['language'];

      $form_state['values']['template_id'] = $node['values']['template_id'];

      // We've done this template, so remove it.
      unset($nodes[0]);
    }

    $parent_node = array(
      'values' => array(
        'uid' => $user->uid,
        'name' => $user->name,
        'title' => $form_state['values']['title'],
        'type' => 'book',
        'body' => $form_state['values']['body'],
        'tokens' => $form_state['values']['tokens'],
        'skeleton_id' => $form_state['values']['skeleton_id'],
        'template_id' => $form_state['values']['template_id'],
        'op' => t('Save'),
      ),
    );

    if (module_exists('translation')) {
      $parent_node['values']['language'] = $form_state['values']['language'];
    }

    // Copy CCK fields.
    if (module_exists('content')) {
      $content_type = content_types('book');
      $fields = $content_type['fields'];
      if (is_array($fields)) {
        foreach ($fields as $field) {
          if (isset($node['values']->{$field['field_name']})) {
            $parent_node['values'][$field['field_name']] = $node['values'][$field['field_name']];
          }
        }
      }
    }

    $parent_node['values']['book']['bid'] = 'new';
    $parent_node['values']['book']['options'] = array();
    drupal_execute('book_node_form', $parent_node, $parent_node['values']);
    // TODO: This will probably need to be removed if this call is integrated
    // into the book module. See http://drupal.org/node/364529 for details.
    menu_rebuild();
    $parent_node = node_load($parent_node['nid'], NULL, TRUE);

    // Only do this if pathauto exists, as it involves another save slowing
    // things down.  
    if (module_exists('pathauto')) {
      $parent_node->title = str_replace('[\skeleton-bookpathalias\]', drupal_get_path_alias($parent_node->path), $parent_node->title);
      $parent_node->teaser = str_replace('[\skeleton-bookpathalias\]', drupal_get_path_alias($parent_node->path), $parent_node->teaser);
      $parent_node->body = str_replace('[\skeleton-bookpathalias\]', drupal_get_path_alias($parent_node->path), $parent_node->body);
      $content_type = content_types($parent_node->type);
      $fields = $content_type['fields'];
      foreach ($fields as $field) {
        if ($field['type'] == 'text' && is_array($parent_node->{$field['field_name']})) {
          foreach (array_keys($parent_node->{$field['field_name']}) as $delta) {
            $parent_node->{$field['field_name']}[$delta]['value'] = str_replace('[\skeleton-bookpathalias\]', drupal_get_path_alias($parent_node->path), $parent_node->{$field['field_name']}[$delta]['value']);
          }
        }
      }
      $parent_node->skeleton_template->keep_connected = TRUE;
      $parent_node = node_submit($parent_node);
      node_save($parent_node);
    }

    // get the id we just created
    $form_state['nids'][] = $parent_node->nid;
    $book_id = $parent_node->nid;
    $goto_id = $book_id;

    // Iterate through the dummy nodes and create the book pages.
    // The Book ID is the NID of the topmost book page.
    foreach ($nodes as $data) {
      $node = array();
      $node['values'] = unserialize($data->node_data);
      // if parents are assigned by the skeleton, use them
      if ($data->parent > 0 && isset($parents[$data->parent])) {
        $parent_node = node_load($parents[$data->parent], NULL, TRUE);
      }
      else {
        $parent_node = node_load($book_id, NULL, TRUE);
      }
      $node['values']['book'] = array();
      $node['values']['book']['bid'] = $book_id;
      $node['values']['book']['plid'] = $parent_node->book['mlid'];

      $node['values']['book']['weight'] = $data->weight;
      $node['values']['book']['options'] = array();

      // Replace custom tokens in the title and body fields. Replace skeleton
      // tokens first so they may be used as node tokens such as [title].
      if (!empty($form_state['values']['skeleton_tokens'])) {
        $node['values'] = (array)_skeleton_token_replace_values((object)$node['values'], 'skeleton', $form_state['values']['skeleton_tokens']);
      }

      $node['values'] = (array)_skeleton_token_replace_values((object)$node['values'], 'node', (object)$node['values'], TRUE);

      $node['values']['uid'] = $user->uid;
      $node['values']['name'] = $user->name;

      $node['values']['tokens'] = $form_state['values']['skeleton_tokens'];

      // We have to add the skeleton ID here as we can't save it with the
      // template as a template may have multiple skeletons.
      $node['values']['skeleton_id'] = $form_state['values']['skeleton_id'];

      $node['values']['op'] = t('Save');
      drupal_execute($node['values']['type'] .'_node_form', $node, (object)$node['values']);
      // TODO: This will probably need to be removed if this call is integrated
      // into the book module. See http://drupal.org/node/364529 for details.
      menu_rebuild();
      $form_state['nids'][] = $node['nid'];
      // Store all of the menu id's for each inserted template.
      $parents[$data->template_id] = $node['nid'];
    }
  }
  else {
    form_set_error(t('This skeleton is empty.'), 'form');
  }
  if (!$form_state['values']['title']) {
    $form_state['redirect'] = 'node/' . $goto_id;
  }
  else {
    $form_state['redirect'] = 'node/' . $goto_id . '/edit';
  }
}
